#include "clickEstimator.hh"

using namespace boost;
using namespace boost::numeric;
using namespace std;

ClickEstimator::ClickEstimator( QObject *parent ):
  QObject(parent), delay(6), sensitivity(16), stopped(false), dragging(false),
  clicked(false)
{
}

void ClickEstimator::setPosition( const Vector &v )
{
  positions.push_back( v );
  emit mouseMove( (int)v[0], (int)v[1] );
  while ( positions.size() > delay )
    positions.pop_front();
  if ( positions.size() == delay ) {
    Vector avg( average() );
    double variance = 0.0;
    for ( deque< Vector >::const_iterator i = positions.begin();
          i != positions.end(); i++ ) {
      Vector diff( *i - avg );
      variance += diff[0] * diff[0] + diff[1] * diff[1];
    };
    variance = sqrt( variance / ( positions.size() - 1 ) );
    emit varianceChanged( (int)variance );
    bool newStopped = variance <= sensitivity;
    if ( stopped != newStopped ) {
      if ( newStopped ) {
        if ( dragging ) {
          if ( !clicked )
            emit startDrag();
          else
            emit stopDrag();
          clicked = !clicked;
        } else
          emit click();
      };
      stopped = newStopped;
    };
  };
}

void ClickEstimator::clearPosition(void)
{
  positions.clear();
  stopped = false;
}

void ClickEstimator::setDelay( int _delay )
{
  delay = _delay;
}

void ClickEstimator::setSensitivity( int _sensitivity )
{
  sensitivity = _sensitivity;
}

ClickEstimator::Vector ClickEstimator::average(void)
{
  assert( !positions.empty() );
  Vector retVal( 3 );
  retVal[0] = 0.0; retVal[1] = 0.0;
  for ( deque< Vector >::const_iterator i = positions.begin();
        i != positions.end(); i++ ) {
    retVal[0] += (*i)[0];
    retVal[1] += (*i)[1];
  };
  retVal[2] = 1.0;
  retVal[0] /= positions.size();
  retVal[1] /= positions.size();
  return retVal;
}

void ClickEstimator::setDragging( bool _dragging )
{
  dragging = _dragging;
}

